#!/usr/bin/env python
import os
import argparse

import cv2

import rosbag
from sensor_msgs.msg import Image, CompressedImage
from cv_bridge import CvBridge
import numpy as np

def main():
    """Extract a folder of images from a rosbag.
    """
    parser = argparse.ArgumentParser(description="Extract images from a ROS bag.")
    parser.add_argument("--bag_file", help="Input ROS bag.")
    parser.add_argument("--poses_file", help="Input Poses file (uses timestamps to select images).")
    parser.add_argument("--output_dir", help="Output directory.")
    parser.add_argument("--image_topic", help="Image topic.", default="/camera/rgb/image_color/compressed")
    parser.add_argument("--raw", dest='raw', action='store_true')

    args = parser.parse_args()

    bag = rosbag.Bag(args.bag_file, "r")

    print("Extract images from %s on topic %s into %s" % (args.bag_file,
                                                          args.image_topic, args.output_dir))

    lines = np.loadtxt(args.poses_file, comments="#", delimiter=" ", unpack=False)

    line_times = np.array(lines[:,0])
    bag_times = np.array([t.to_sec() for topic, msg, t in bag.read_messages(topics=[args.image_topic])])

    print("Times: {} vs. {}".format(line_times[0], bag_times[0]))

    try:
        os.mkdir(args.output_dir)
    except OSError:
        print("Folder Already Exists: {}".format(args.output_dir))

    out_file = open(os.path.join(args.output_dir, "metadata.txt"), "w")

    bridge = CvBridge()
    count = 0
    min_offset = np.Inf
    for topic, msg, t in bag.read_messages(topics=[args.image_topic]):

        t = t.to_sec()
        stamp = lines[count][0]

        values = lines[count][1::]
        offset = abs(t - stamp)
        print("{} - {} = {} ({})".format(t, stamp, offset, min_offset))
        if min_offset > offset:
            min_offset = offset
            min_msg = msg
            continue
        else:
            min_offset = np.Inf

            print("Using {}".format(min_msg.header.stamp.to_sec()))
            #### direct conversion to CV2 ####
            if args.raw:
                cv_img = bridge.imgmsg_to_cv2(min_msg, desired_encoding="passthrough")
            else:
                np_arr = np.fromstring(min_msg.data, np.uint8)
                cv_img = cv2.imdecode(np_arr, cv2.IMREAD_ANYCOLOR)

            cv2.imshow("Image",cv_img)
            cv2.waitKey(1)

            filename = os.path.join(args.output_dir, "{:10.4f}.png".format(stamp))
            cv2.imwrite(filename, cv_img)
            print(         "{} {} {} {} {} {} {} {}  ".format(filename, values[0], values[1], values[2], values[6], values[3], values[4], values[5]))
            out_file.write("{} {} {} {} {} {} {} {}\n".format(filename, values[0], values[1], values[2], values[6], values[3], values[4], values[5]))

            count += 1

            if count >= lines.shape[0]:
                break

    out_file.close()
    bag.close()

    return

if __name__ == '__main__':
    main()
